package salleVente;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import java.io.*;
import java.util.Date;

import SalleEnchere.*;
import structureEncheres.*;
import NommageFacile.*;
import easyNotify.*;

/**
 * Cette classe reprsente le serveur corba.
 * celui-ci se charge de rendre disponible l'objet sur l'orb,
 * de le faire rfrencer auprs du service de nommage,
 * puis d'obtenir un canal auprs du service de notification.
 * 
 * @author Bencherif, Kartasasmita, Mathon, Parra dP.
 */

public class Server {

    /** le chemin d'accs  la salle pour le service de nommage **/
    private final static String CONTEXTESALLE = "/sallesDeVentes/salle2";

    /** ports et machines par dfaut pour les services de nommage et de notification **/
    private final static String HOST_NOMMAGE = "localhost";
    private final static String HOST_NOTIFICATION = "localhost";
    private final static int PORT_NOMMAGE = 10000;
    private final static int PORT_NOTIFICATION = 10001;

    // ---------------------------------------------------------------------------------

    /**
     * mthode qui execute la salle de vente :
     *   - cre l'objet de la salle de vente
     *   - le rend disponible auprs de l'orb
     *   - le rfrence auprs du service de nommage
     *   - obtient un canal vers lequel envoyer des evenements, auprs du service de notification
     * @param orb
     * @param machineNamingService
     * @param portNamingService
     * @param machineNotifyService
     * @param portNotifyService
     **/
    static SalleEnchereImpl runSalledeVente(ORB orb, 
					    String machineNamingService, int portNamingService,
					    String machineNotifyService, int portNotifyService) {

        // 1) on rcupre un canal auprs du service de notification
	EasyNotifyStructuredPushSupplier notifyTmp = null;
	try {
	    notifyTmp = new EasyNotifyStructuredPushSupplier(orb,  machineNotifyService, Integer.toString(portNotifyService));

	} catch (Exception e) {
	    System.out.println("\n\nimpossible de communiquer avec le service de Notification!!!\n\n message d'exception gnr : \n");
	    e.printStackTrace();
	}
	
        // 2) on cre l'objet avec ses paramtres par dfaut
	SalleEnchereImpl salleEnchereImpl = new SalleEnchereImpl(notifyTmp, orb, new Date((new Date()).getTime() + ParametresSalle.DUREE_AVANT_ENCHERE), ParametresSalle.PAS_BAISSE, ParametresSalle.DUREE_TOP);
	SalleVente salleDeVente = salleEnchereImpl._this(orb);

	// 3) on fait rfrencer l'objet auprs du service de nommage
	String nomObjet = new String(machineNamingService+":"+portNamingService+":"+CONTEXTESALLE);
	EasyNaming.inscriptionObjet(orb, nomObjet, salleDeVente);

	return salleEnchereImpl;
    }


    // ---------------------------------------------------------------------------------

    static void stopSalledeVente(ORB orb, String machineNamingService, int portNamingService, SalleEnchereImpl salleEnchereImpl) {
	// stoppe la salle de vente
	// (note:  la dsinscription auprs du service de notification est effectu par la salle de vente en elle-mme)
	salleEnchereImpl.stoppeTout();
	// se dsinscrit auprs du service de nommage
	String nomObjet = new String(machineNamingService+":"+portNamingService+":"+CONTEXTESALLE);
	EasyNaming.desinscriptionObjet(orb, nomObjet);
    }


    // ---------------------------------------------------------------------------------

    static void run(ORB orb, 
		    String machineNamingService, int portNamingService,
		    String machineNotifyService, int portNotifyService) {
	try {
	    org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
	    POA rootPOA = POAHelper.narrow(obj);
	    POAManager manager = rootPOA.the_POAManager();
	    
	    // effectue les initialisation pour un objet de la salle de vente
	    SalleEnchereImpl salleEnchereImpl 
		= runSalledeVente(orb, machineNamingService, portNamingService, machineNotifyService, portNotifyService);

	    manager.activate();
	    orb.run();

	    stopSalledeVente(orb, machineNamingService, portNamingService, salleEnchereImpl);

	} catch (Exception e) {
	    e.printStackTrace();
	    System.exit(1);
	}
    }


    // ---------------------------------------------------------------------------------

    /**
     * mthode principale
     */
    public static void main (String[] args) {

	String hostNom = HOST_NOMMAGE, hostNotif = HOST_NOTIFICATION;
	int portNom = PORT_NOMMAGE, portNotif = PORT_NOTIFICATION;
	switch (args.length) {
	case 0 : break;
	case 4 : 
	    portNom = (new Integer(args[2]).intValue());
	    portNotif = (new Integer(args[3]).intValue());
	case 2 : 
	    hostNom = args[0];
	    hostNotif = args[1];
	    break;
	default :
	    System.out.println("USAGE :\n   Server [Host_Service_Nommage Host_Service_Notification [Port_ServiceNommage Port_Service_Notification]]\n\n  => utiliser le nom de la machine du NamingService, puis la machine du service de notification, \n     puis leur ports respectifs si ncessaire.\n     Par dfaut, le NamingService et le NotifcationService sont en localhost \n     et utilisent respectivement les ports 10000 et 10001\n");
	    System.exit(1);
	    break;
	} // end switch
	
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass", "com.ooc.CORBA.ORBSingleton");
	
	ORB orb = ORB.init(args, props);
	run(orb, hostNom, portNom, hostNotif, portNotif);
    } // end main()
}
