package salleVente;

import java.util.*;
import java.io.*;
import org.omg.CORBA.*;
import org.omg.CosNotifyChannelAdmin.AdminLimitExceeded;     // pour les exceptions du service de Notification
import org.omg.CosEventChannelAdmin.AlreadyConnected;

import SalleEnchere.*;
import structureEncheres.*;

import SalleEnchere.SalleVentePackage.*; // parceque jidl gnre les exceptions dans ce package au nom trs explicite...
import NommageFacile.*;
import easyNotify.*;


/**
 * Cette classe est l'implmentation de la salle de vente.
 * 
 * @author Bencherif, Kartasasmita, Mathon, Parra dP.
 */

public class SalleEnchereImpl extends SalleVentePOA {

  // -----------------------------------------------------
  //
  // Attributs
  //

    // par rapport  la salle et l'ensemble des ventes
    private Catalogue catalogueVente;
    private String fichierCatalogue;  // le fichier du catalogue
    private Date dateDebutEnchere;
    private Date dateFinEnchere;
    private EasyNotifyStructuredPushSupplier eventNotify;    // infos d'enregistrement  l'event Service
    private EventServiceId infoServiceNotify;  // info du service de notification pour l'encherisseur qui entre dans la salle (canal, port & host machine)
    private ORB orb;   // l'orb (utilis uniquement pour gnrer un vnement du service de notification)
    private CommissairePriseur horloge;

    // par rapport  l'objet en cours d'enchre
    private Enchere enchereCourante;  // enchere en cours.
    private int pasEnchere;           // pas de l'enchre (pour baisser le prix)
    private int numeroproduitDansLeCatalogue;
    private static final int prixEnchereMinimal = 2;

    // par rapport  l'identification des personnes.
    private Vector listePersonnes;                // la cl est le numero, la valeur est la Personne
    private int prochainIdPersonne = 0;           // le prochain identifiant allou  une personne qui entre dans la salle
    private static final int idCommissaire = -1;  // l'identifiant du commissaire priseur
    private static final String nomCommissaire = "le commissaire priseur";


  // -----------------------------------------------------
  //
  // Constructeurs
  //

    /** note : on passe en paramtres au constructeur :
     *                 - une rfrence sur l'eventNotify (parceque la rcupration du canal se fait depuis le serveur, avant de lancer l'orb)
     *                 - les informations de canal, port, et host de la machine o se situe le notify event service
     *                   (parceque l'encherisseur rcupre son canal d'coute du notify service lors de son entre dans la salle d'enchre).
     *                   l'IDL dfinit la structure EventServiceId. D'o le paramtre.
     *                 - l'ORB (uniquement utile pour l'venement gnr pour le service de notification)
     *                 - la date(et heure) de dbut d'enchre
     *                 - le pas d'enchre pour cette salle
     *                 - le dlai d'un TOP (une fois, deux fois, etc.)
     */
    public SalleEnchereImpl(EasyNotifyStructuredPushSupplier eventNotify, ORB orb,
			    Date dateDebut, int pasEnchere, long delaiTop, String fichierProduits) {
	fichierCatalogue = fichierProduits;
	this.orb = orb;
	this.eventNotify = eventNotify;
	// on cre un objet EventServiceId (contenant les infos pour accder au service de notification)
	// cet objet est envoy  chaque pesonne qui entre dans la salle de vente.
	if (eventNotify != null) {
	    infoServiceNotify = new EventServiceId(eventNotify.getChannelId(), Integer.parseInt(eventNotify.getMachinePort()), eventNotify.getMachineName());
	    System.out.println("\n\nUtilisation du service de notification :  port=" + infoServiceNotify.numPort + "  -  host=" + infoServiceNotify.nomMachine + "  -  canal=" + infoServiceNotify.numCanal);
	} else {
	    System.out.println("\n\nATTENTION : service de notification inexploitable. \nAucun acheteur ne pourra tre correctement averti!");
	}
        dateDebutEnchere = dateDebut;
	System.out.println("\n------ Dbut des enchres dans " + ((dateDebut.getTime() - (new Date()).getTime()) / 1000) + " secondes ------");
        dateFinEnchere = null;   // enchere non finie.
	listePersonnes = new Vector();
        this.pasEnchere = pasEnchere;
	initialiseCatalogue();
	// initialisation de l'enchere courante (aucune par dfaut)
        enchereCourante = new Enchere();
	enchereCourante.leStatut = Statut.enBaisse;  // sinon le POA ne gre pas correctement la traduction.
        numeroproduitDansLeCatalogue = -1;
	horloge = new CommissairePriseur(delaiTop, dateDebutEnchere.getTime() - (new Date()).getTime() );
	horloge.start();
    }

    public SalleEnchereImpl(EasyNotifyStructuredPushSupplier eventNotify, ORB orb,
			    Date dateDebut, int pasEnchere, long delaiTop) {
	this(eventNotify, orb, dateDebut, pasEnchere, delaiTop, "produits.cat");
    }



  // -----------------------------------------------------
  //
  // oprations de SallePOA redfinies pour que le client entre/sorte.
  //

    /** 
     * entree d'une personne dans la salle (mthode dfinie dans l'IDL)
     * @param p
     * @param idPersonne
     * @param idEV
     */
    public Enchere entree(Personne p,
			  org.omg.CORBA.IntHolder idPersonne,
			  EventServiceIdHolder idEV)
	throws SalleFermee {
	if (dateFinEnchere == null) { // enchere non finie
	    System.out.println("\n une nouvelle personne entre (id=" + prochainIdPersonne + ")" + PersonneToString(p));
	    idPersonne.value = prochainIdPersonne;
	    listePersonnes.add(p);
	    // on lui renvoie l'identifiant de service Notify  l'utilisateur.
	    idEV.value = infoServiceNotify;
	    prochainIdPersonne++;
	    return enchereCourante;
	} else {
	    throw (new SalleFermee("salle ferme depuis " + ((new Date()).getTime() - dateDebutEnchere.getTime())/600 + " minutes."));
	}
	
	// renvoie le produit en cours, son prix, et le status 
    }


    // --------------------------------------------------------------------------------------- //

    /** 
     * rcupration du catalogue de la vente (mthode dfinie dans l'IDL)
     */
    public synchronized Catalogue getCatalogue(){
	// quelqu'un souhaite le catalogue : on modifie la valeur de dbut d'enchre.
	long diff = ((new Date()).getTime() - dateDebutEnchere.getTime());
	if (diff > 0)
	    catalogueVente.tempsRestant = 0;
	else
	    catalogueVente.tempsRestant = (int)diff;
	System.out.println("\n quelqu'un rcupre le catalogue...");
	return catalogueVente;
    }


    // --------------------------------------------------------------------------------------- //

    /** 
     * encherissement sur l'objet en cours (mthode dfinie dans l'IDL)
     * @param idPersonne
     * @param idProduit
     * @param prixEnchere
     */
    public synchronized void encherir (int idPersonne,
				       int idProduit,
				       int prixEnchere)
	throws EnchereBasse, MauvaisProduit {
	
	if (idProduit != enchereCourante.idProduitCourant) {
	    System.out.println("... "+idPersonne+" a tent d'encherir sur un produit qui n'est pas le bon !");
	    throw(new MauvaisProduit("trop tard : produit dj vendu!"));
	} else if ((idPersonne != idCommissaire) && (prixEnchere <= enchereCourante.prixCourant)) {
	    System.out.println("... "+idPersonne+" a tent d'encherir mais n'est pas suffisammet gnreux (prix trop bas) !");
	    throw(new EnchereBasse("trop tard : prix dj atteint!"));
	} else try {  // on a un nouvel encherisseur
	    Personne pers = (Personne)listePersonnes.get(idPersonne);
	    enchereCourante.leStatut = Statut.encheri;
	    enchereCourante.prixCourant = prixEnchere;
	    enchereCourante.idPersonne = idPersonne;
	    // on avertit l'horloge de dbuter un nouveau top
	    System.out.println("\n... OUI !  quelqu'un a enchri.");
	    System.out.println("\n "+ pers.prenom + " " + pers.nom + " (id=" + idPersonne + ") encherit  " + prixEnchere + "sur l'objet n" + idProduit);
	    try {
		if (eventNotify != null)
		    eventNotify.push(new SdvEvent(orb, enchereCourante, new String(((Personne)listePersonnes.get(enchereCourante.idPersonne)).prenom + " " + ((Personne)listePersonnes.get(enchereCourante.idPersonne)).nom), "quelqu'un a encheri"));
		horloge.notify();  // rveille le thread s'il tait dans une phase d'attente.
	    } catch (org.omg.CosEventComm.Disconnected e) {
		System.out.println("... Impossible d'avertir les clients : le service de notification vient de vous dconnecter!");
		eventNotify = null;
	    } catch (IllegalMonitorStateException e) {  // sinon il tait dj rveill.
	    }

	} catch (ArrayIndexOutOfBoundsException e) {
	    System.out.println("... "+idPersonne+" a tent d'encherir alors qu'il n'est pas entr dans la salle !");
	    throw(new MauvaisProduit("vous ne pouvez pas encherir. Entrez d'abord dans la salle !"));
	}
    }


    // --------------------------------------------------------------------------------------- //

    /** 
     * une personne sort de la salle (mthode dfinie dans l'IDL)
     * @param idPersonne
     */
    public void sortie(int idPersonne) {
	try {
	    listePersonnes.remove(idPersonne);
	    System.out.println("\n " + idPersonne + " sort.");
	} catch (ArrayIndexOutOfBoundsException e) {
	    System.out.println("\n " + idPersonne + " tente de sortir alors qu'elle n'est pas encore entre !");
	}
    }


  // -----------------------------------------------------
  //
  // oprations internes  la salle de vente
  //

    /** 
     * top du marteau (dclench par l'horloge du "commissaire priseur").
     *   - change le statut de l'enchre.
     *   - prvient le service de notification du changement de statut.
     */
    private synchronized void top() {
        String messageTmp = "";
        synchronized(enchereCourante) {
            if (enchereCourante.leStatut == Statut.encheri) {
		System.out.println("\n...  la une");
		enchereCourante.leStatut = Statut.uneFois;
		messageTmp = new String("Qui propose un meilleur prix ?");
	    } else if (enchereCourante.leStatut == Statut.uneFois) {
		System.out.println("\n...  la deux");
		enchereCourante.leStatut = Statut.deuxFois;
		messageTmp = new String("Allez mesdames, messieurs, faites votre offre !");
	    } else if (enchereCourante.leStatut == Statut.deuxFois) {
		System.out.println("\n...  la trois !");
		enchereCourante.leStatut = Statut.troisFois;
		messageTmp = new String("Attention, vous ratez une belle affaire ! Encherissez voyons !");
	    } else if (enchereCourante.leStatut == Statut.troisFois) { // adjug vendu
		System.out.println("\n... Adjug vendu !");
		messageTmp = new String("Quel dommage !");
		enchereCourante.leStatut = Statut.adjuge;
		catalogueVente.produits[numeroproduitDansLeCatalogue].vendu = true;
		catalogueVente.produits[numeroproduitDansLeCatalogue].prixVente = enchereCourante.prixCourant;
                // on laisse le temps  l'utilisateur de "savourer" son enchre...
            } else if (enchereCourante.leStatut == Statut.adjuge) { // on change d'enchre
		messageTmp = new String("nous passons  l'enchre suivante.");
                enchereSuivante();
	    } else if (enchereCourante.leStatut == Statut.enBaisse) { // on baisse l'enchre
                enchereCourante.prixCourant -= pasEnchere;
		if ((enchereCourante.prixCourant < (int)(ParametresSalle.RAPPORT_PLANCHER * (float)catalogueVente.produits[numeroproduitDansLeCatalogue].prixDepart))
		     || (enchereCourante.prixCourant < prixEnchereMinimal)) {
		    catalogueVente.produits[numeroproduitDansLeCatalogue].prixVente = -1;
		    System.out.println("\n...  Desol! le prix est trop bas. L'enchre est annule.");
		    messageTmp = new String("Desol! le prix est trop bas. L'enchre est annule.On passe  la suivante.");
		    enchereSuivante();  // NON vendu
		} else {
		    System.out.println("\n...  PAF !  le prix diminue et passe  " + enchereCourante.prixCourant);
		    messageTmp = new String("Allons, faites un effort! Regardez la qualit des finitions !");
		}

	    }

	    // ON AVERTIT TOUS LES CLIENTS DU CHANGEMENT DE STATUS (ou d'enchre le cas chant).
	    try {
		if (eventNotify != null) {
		    if (enchereCourante.idPersonne == idCommissaire)
			eventNotify.push(new SdvEvent(orb, enchereCourante, nomCommissaire, messageTmp));
		    else
			eventNotify.push(new SdvEvent(orb, enchereCourante, new String(((Personne)listePersonnes.get(enchereCourante.idPersonne)).prenom + " " + ((Personne)listePersonnes.get(enchereCourante.idPersonne)).nom), messageTmp));
		    System.out.println("... Les clients sont prvenus changement de la nouvelle action");
		}
	    } catch (org.omg.CosEventComm.Disconnected e) {
		System.out.println("... Impossible d'avertir les clients : le service de notification vient de vous dconnecter!");
		eventNotify = null;
	    }
        }; // fin du synchronized
    }


    // --------------------------------------------------------------------------------------- //

    /** 
     * passe  l'enchre suivante.
     * si tous les objets ont t vendus, se dconnecte du service de notification.
     */
    private synchronized void enchereSuivante() { // passe  l'enchre suivante
        numeroproduitDansLeCatalogue++;
	if (numeroproduitDansLeCatalogue < catalogueVente.produits.length) {
	    System.out.println("\n------ on passe  l'enchre suivante ------\n");
	    System.out.println("\n" + ProduitToString(catalogueVente.produits[numeroproduitDansLeCatalogue]));
	    enchereCourante.idProduitCourant = catalogueVente.produits[numeroproduitDansLeCatalogue].id;
	    enchereCourante.prixCourant = catalogueVente.produits[numeroproduitDansLeCatalogue].prixDepart;
	    enchereCourante.leStatut = Statut.enBaisse;
	    enchereCourante.idPersonne = idCommissaire;
	    // on avertit l'horloge de dbuter un nouveau top
	    try {
		horloge.notify();  // rveille le thread s'il tait dans une phase d'attente.
	    } catch (IllegalMonitorStateException e) {  // sinon il tait dj rveill.
	    }
	} else {   // toutes les enchres sont termines !!
	    System.out.println("\n------ toutes les enchres sont termines... ------\n Au revoir!");
	    dateFinEnchere = new Date();
	    enchereCourante = null;
            stoppeTout();  // stoppe l'horloge et se dsinscrit du service de notification
	}
    }

    // --------------------------------------------------------------------------------------- //

    /** remplit le catalogue de quelques produits ( partir d'un fichier) */
    private void initialiseCatalogue() {
        catalogueVente = new Catalogue();
	catalogueVente.salleDeVente = "Salle de ventes d'objets de luxe uniquement pour Noel.";
	Vector listeProduitsTmp = new Vector();
	try {
	    BufferedReader fichier = new BufferedReader(new FileReader(fichierCatalogue));
	    String ligneLue = fichier.readLine();
	    Produit produitTmp;
	    StringTokenizer token;

	    while (ligneLue != null) {   // un nouveau produit
		token = new StringTokenizer(ligneLue, "#");
		try {
		    if (token.countTokens() == 4) {  // 4 lments :  ID, nom, description, prix de Dpart
		// sinon ligne mal formate : on ne la lit pas
		        produitTmp = new Produit();
			produitTmp.id = Integer.parseInt(token.nextToken());
			produitTmp.nom = token.nextToken();
			produitTmp.description = token.nextToken();
 			produitTmp.prixDepart = Integer.parseInt(token.nextToken());
			produitTmp.vendu = false;
			produitTmp.prixVente = 0;
			// tout est ok : on ajoute un nouveau produit.
			listeProduitsTmp.add(produitTmp);
		    }
		} catch (NumberFormatException e) {
		    // on attend un entier : ce n'en est pas un : donc ligne mal formate!  On ne l'ajoute pas.
		}
		ligneLue = fichier.readLine();    // on passe  la ligne suivante
	    }
	    fichier.close();
	    
	} catch(FileNotFoundException e) {  // capture des exceptions de lecture du fichier.
	    System.out.println("le fichier " + fichierCatalogue + " n'existe pas!");
	    System.exit(1);
	} catch(IOException e) {
	    System.out.println("erreur d'entre/sortie lors de la lecture du fichier " + fichierCatalogue + "!");
	    System.exit(1);
	}

	// puisque listeProduitsTmp.toArray ne marchent pas parceque java utilise des vecteurs d'objets et pas de produits,
	// on est oblig de dfinir un tableau de "Produit" et de faire des cast. Merci l'IDL ;-)
	catalogueVente.produits = new Produit[listeProduitsTmp.size()];
	for (int i=0; i<listeProduitsTmp.size(); ++i)
	    catalogueVente.produits[i] = (Produit)listeProduitsTmp.get(i);
	System.out.println("\nCatalogue cr avec " + catalogueVente.produits.length + " produits!");
    }


    // --------------------------------------------------------------------------------------- //

    /** stoppe l'horloge et se dsinscrit du service de notification */
    void stoppeTout() {
	horloge.stoppe();   // stoppe l'horloge : les enchres sont termines!!!!!
	if (eventNotify != null) {
	    try {  // on avertit les clients de la fin de l'enchre
		enchereCourante = new Enchere();
		enchereCourante.leStatut = Statut.enBaisse;
		eventNotify.push(new SdvEvent(orb, enchereCourante, nomCommissaire, "FERMETURE"));
		System.out.println("... Fermeture de la salle.");
	    } catch (org.omg.CosEventComm.Disconnected e) {
		System.out.println("... Impossible d'avertir les clients : le service de notification vient de vous dconnecter!");
		eventNotify = null;
	    }
	    eventNotify.disconnect();  // se dconnecte du service de notification
	    eventNotify = null;
	    System.out.println("... Deconnection du service de notification.");
	}
    }


    // mthodes lies aux structures dfinies dans l'IDL.
    // on part du principe qu'on a pas  y toucher vu que c'est gnr par jidl...
    // (ce qui est un peu dommage vu qu'on ne fait plus vraiment de l'objet ;-)

    /** redefinition de toString() pour une Personne */
    private String PersonneToString(Personne p) {
      return("\n   Personne: "+p.prenom+" "+p.nom+"\n");
    }

    /** redefinition de toString() pour un Produit */
    private String ProduitToString(Produit p) {
	return("\n\nProduit n. " + p.id + ": \n   nom: " + p.nom + "\n   description: " + p.description + "\n   prix de dpart: " + p.prixDepart + "\n");
    }

    // +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
    // +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
    // +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

    /**
     * classe interne : gestion du commissaire priseur (l'horloge qui gre les tops)
     */
    public class CommissairePriseur extends Thread{
        private long dureeTop;           // duree d'un top en millisecondes
        private long debutEnchere;       // temps avant le debut de l'enchre
	private boolean enchereTerminee; // indique si l'enchre est termine

	public CommissairePriseur(long duree, long dureeAvantDebutEnchere) {
	    super();
	    enchereTerminee = false;
            dureeTop = duree;
	    debutEnchere = dureeAvantDebutEnchere;
	}

    // --------------------------------------------------------------------------------------- //

	/** execution du thread */
	public void run() {
	    try {
		if (eventNotify != null)
		    eventNotify.connect();
		else
		    System.out.println("\nATTENTION : pas de service de notification exploitable. La salle de vente n'aura probablement aucun acheteur..."); 		if (debutEnchere > 0)
		    sleep(debutEnchere);
		enchereSuivante();
	    } catch (AdminLimitExceeded e) {
		System.out.println("ATTENTION : impossibilit de connection au service de notification!!!");
		return;  // problme  la connection au Notify Service.
	    } catch (AlreadyConnected e) {
		System.out.println("ATTENTION : dj connect au service de notification!!!");
		return;  // problme  la connection au Notify Service.
	    } catch (InterruptedException e) {
		return;  // fin du processus
	    } catch (IllegalMonitorStateException e) {
	       // rien
	    }
	    while(!enchereTerminee) { // on boucle infiniment
		// attend le dbut de l'enchre
		// attend et fait un top
		coupMarteau();
	    }
	    return; // fin du thread
	}


    // --------------------------------------------------------------------------------------- //

	/** un coup de marteau : consiste  attendre un certain dlai puis  prvenir la salle de vente du top */
	public synchronized void coupMarteau() {
	    try {
                wait(dureeTop);  // dure du top
                top();  // fait un top
		return; // fin du processus
	    } catch (InterruptedException e) {
                top();  // fait un top
		return;
	    }
	}


    // --------------------------------------------------------------------------------------- //
	/** indique au thread de s'arrter */
	public synchronized void stoppe() {
	    enchereTerminee = true;
	    try {
		notify(); // rveille le thread s'il tait dans une phase d'attente.
	    } catch (IllegalMonitorStateException e) {  // sinon il tait dj rveill.
	    }
	}


    } // fin CommissairePriseur

} // fin SalleEnchereImpl
